import json as og_json
import datetime
from json import *
from collections.abc import Mapping, Iterable


"""
99% of the time, if you do the import like this:

import giga_json as json

you can use it 100% identically to the standard json module like this:

json.dumps(some_dict)

what's different then?  two things.
 
first thing: defaults to dumps(indent=4, sort_keys=True)

second thing: it uses a custom serializer, appropriately named GigaEncoder, to handle the serialization of most common 
python objects to help avoid the annoying errors about objects that aren't json serializable.  not being able to do 
json.dumps() on a dictionary or list containing datetime drove me crazy.  but that's just the tip of the iceberg.  see
the https://github.com/nebko16/giga_json for the full details and examples
"""


class GigaEncoder(og_json.JSONEncoder):

    def __init__(self, *args, raise_on_error=False, debug=False, **kwargs):
        super().__init__(*args, **kwargs)
        self.raise_on_error = raise_on_error
        self.debug = debug

    def default(self, o):
        if self.debug:
            print(f"actual type: {type(o)}")

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # date and datetime

        object_type_name = type(o).__name__
        if isinstance(o, (datetime.date, datetime.datetime)):
            if self.debug:
                print(f"match: datetime")

            try:
                return o.isoformat()
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # requests support

        elif object_type_name == 'Response':
            if self.debug:
                print(f"match: requests Response")

            try:
                jdata = o.json()
                jdata['status_code'] = o.status_code
                jdata['reason'] = o.reason
                jdata['headers'] = o.headers
                return jdata
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # flask support

        elif object_type_name == 'Request':
            if self.debug:
                print(f"match: flask request")

            try:
                url = o.url
                http_method = o.method
                headers = dict(o.headers)
                user_agent = o.headers.get('User-Agent')
                ip_address = o.remote_addr
                content_type = o.headers.get('Content-Type')

                if o.method == 'POST':
                    if content_type == 'application/json':
                        body = o.json
                    elif content_type == 'application/x-www-form-urlencoded':
                        body = o.form
                    elif content_type == 'multipart/form-data':
                        body = o.form
                    else:
                        body = None
                else:
                    body = o.args

                important_data = {
                    'url': url,
                    'http_method': http_method,
                    'headers': headers,
                    'user_agent': user_agent,
                    'ip_address': ip_address,
                    'body': body
                }
                return important_data
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # MatPlotLib support

        elif object_type_name == 'Axes':
            if self.debug:
                print(f"match: MatPlotLib Plot")

            try:
                plot_data = [{'x': line.get_xdata().tolist(), 'y': line.get_ydata().tolist()} for line in o.get_lines()]
                return plot_data
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # SciPy support

        elif object_type_name == 'csr_matrix':
            if self.debug:
                print(f"match: SciPy sparse matrix")

            try:
                return o.toarray().tolist()
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # PyTorch support

        elif object_type_name == 'Tensor':

            if self.debug:
                print(f"match: PyTorch tensor")

            try:
                return o.tolist()
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # TensorFlow support

        elif object_type_name == 'EagerTensor':
            if self.debug:
                print(f"match: TensorFlow tensor")

            try:
                return o.numpy().tolist()
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # Pandas support

        elif object_type_name == 'DataFrame':
            if self.debug:
                print(f"match: Pandas DataFrame")

            try:
                return o.to_dict(orient='records')
            except (TypeError, ValueError):
                pass

        elif object_type_name == 'Series':
            if self.debug:
                print(f"match: Pandas Series")

            try:
                return o.to_dict()
            except (TypeError, ValueError):
                pass

        elif object_type_name == 'Index':
            if self.debug:
                print(f"match: Pandas Index")

            try:
                return o.tolist()
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # this block checks to see if an object might have its own built-in serialization.  if it does, we use that, as
        # it's likely going to handle serialization better than relying on dunder checks later in our encoder

        elif hasattr(o, 'to_json'):
            if self.debug:
                print(f"match: to_json()")

            try:
                return o.to_json()
            except (TypeError, ValueError):
                pass

        elif hasattr(o, 'json'):
            if self.debug:
                print(f"match: json()")

            try:
                return o.json()
            except (TypeError, ValueError):
                pass

        elif hasattr(o, 'toJSON'):
            if self.debug:
                print(f"match: toJSON()")

            try:
                return o.toJSON()
            except (TypeError, ValueError):
                pass

        elif hasattr(o, 'as_json'):
            if self.debug:
                print(f"match: as_json()")

            try:
                return o.as_json()
            except (TypeError, ValueError):
                pass

        elif hasattr(o, 'get_json'):
            if self.debug:
                print(f"match: get_json()")

            try:
                return o.get_json()
            except (TypeError, ValueError):
                pass

        elif hasattr(o, 'serialize'):
            if self.debug:
                print(f"match: serialize()")

            try:
                return o.serialize()
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # NumPy support

        elif object_type_name == 'recarray':
            if self.debug:
                print(f"match: numpy recarray")

            try:
                return [dict(zip(o.dtype.names, record)) for record in o]
            except (TypeError, ValueError):
                pass

        elif object_type_name == 'MaskedArray':
            if self.debug:
                print(f"match: numpy masked array")

            try:
                return {'data': o.data.tolist(), 'mask': o.mask.tolist()}
            except (TypeError, ValueError):
                pass

        elif object_type_name == 'ndarray':
            if self.debug:
                print(f"match: numpy ndarray")

            try:
                return o.tolist()
            except (TypeError, ValueError):
                pass

        elif object_type_name == 'int32':
            if self.debug:
                print(f"match: numpy number")

            try:
                return o.item()
            except (TypeError, ValueError):
                pass

        elif object_type_name == 'Int64DType':
            if self.debug:
                print(f"match: numpy dtype")

            try:
                return str(o)
            except (TypeError, ValueError):
                pass

        elif object_type_name == 'matrix':
            if self.debug:
                print(f"match: numpy matrix")

            try:
                return o.tolist()
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        elif isinstance(o, (bytearray, set, frozenset, range)):
            if self.debug:
                print(f"match: (bytearray, set, frozenset, range)")

            try:
                return list(iter(o))
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        elif isinstance(o, memoryview):
            if self.debug:
                print(f"match: memoryview")

            try:
                return o.tobytes().decode('utf-8')
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        elif object_type_name == 'Decimal':
            if self.debug:
                print(f"match: Decimal")

            try:
                return float(o)
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        elif object_type_name == 'UUID':
            if self.debug:
                print(f"match: UUID")

            try:
                return str(o)
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # named tuple

        elif hasattr(o, '_asdict'):
            if self.debug:
                print(f"match: _asdict()")

            try:
                return o._asdict()
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # check for K:V types first, since they also meet the criteria for iterables, which would only take the keys

        elif isinstance(o, Mapping):
            if self.debug:
                print(f"match: Mapping")

            try:
                return dict(o)
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        elif isinstance(o, bytes):
            if self.debug:
                print(f"match: bytes")

            try:
                return o.decode('utf-8')
            except UnicodeDecodeError:
                import base64
                try:
                    return base64.b64encode(o).decode('ascii')
                except (TypeError, ValueError):
                    pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        elif object_type_name == 'EnumMeta':
            if self.debug:
                print(f"match: Enum")

            try:
                return o.value
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        elif isinstance(o, complex):
            if self.debug:
                print(f"match: complex")

            try:
                r, i = decimal_trunc(o.real), decimal_trunc(o.imag)
                s = f' + {i}i' if i >= 0 else f' - {i*-1}i'
                return f"{r}{s}"
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # objects that don't extend built-in types, and don't have _asdict(),  but have iter and getitem

        elif is_dict_like(o):
            if self.debug:
                print(f"match: dict-like custom object")

            try:
                return dict(o.items() if hasattr(o, 'items') else o)
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # objects you can directly iterate. we do this last because many types actually fit this criteria

        elif (isinstance(o, Iterable) and not isinstance(o, str)) or hasattr(o, '__iter__'):
            if self.debug:
                print(f"match: Iterable or has __iter__")

            try:
                return list(iter(o))
            except (TypeError, ValueError):
                pass

        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # if all other serialization attempts either didn't match or didn't succeed, we either raise, if raise on error
        # flag is set to True, otherwise we return json null

        else:
            if self.debug:
                print(f"match: else")

            if self.raise_on_error:
                return super().default(o)
            else:
                try:
                    return str(o)
                except (TypeError, ValueError):
                    return None


def is_dict_like(obj):
    return hasattr(obj, '__getitem__') and hasattr(obj, '__iter__')


def decimal_trunc(num):
    return int(num) if num == int(num) else num


def og_dumps(obj, *, indent=None, sort_keys=False, **kwargs):
    """
    easy access to the vanilla dumps for those that need or want it.  this is literally the
    vanilla json.dumps(), but with a different name.  it's here purely for convenience
    """
    return og_json.dumps(obj, indent=indent, sort_keys=sort_keys, **kwargs)


def flat_dumps(obj, *, debug=False, raise_on_error=False, indent=None, sort_keys=False, **kwargs):
    """
    do you need the flexibility of giga_json's GigaEncoder serializer, but need the output to be flat (no line breaks or
    indents)?  if yes, this is exactly what that.  better serialization flexibility but with vanilla output formatting
    """
    return og_json.dumps(obj, cls=GigaEncoder, debug=debug, raise_on_error=raise_on_error, indent=indent, sort_keys=sort_keys, **kwargs)


def dumps(obj, *, debug=False, raise_on_error=False, indent=4, sort_keys=True, **kwargs):
    """
    this one is why you're here.  the other functions are for convenience when needed, but this one is the magic sauce
    """
    return og_json.dumps(obj, cls=GigaEncoder, debug=debug, raise_on_error=raise_on_error, indent=indent, sort_keys=sort_keys, **kwargs)



'''
Picklete
'''


import os
from setuptools import setup, find_packages

setup(
    name='giga_json',
    version='1.0.1',
    author='nebko16',
    author_email='nebko16@gmail.com',
    description=("extension of python json module, but dumps pretty prints by default and the serializer can handle almost any python object you give it"),
    license='GPL-3.0',
    keywords='python python3 json giga gigajson giga_json module pprint dumps indent prettyprint pretty print format serializer',
    packages=find_packages(),
    install_requires=[],
    python_requires='>=3.6',
    url='https://github.com/nebko16/giga_json'
)



import sys
import os
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
import unittest
import datetime
import uuid
import giga_json as json
try:
    import torch
except ImportError:
    torch = None
try:
    import numpy as np
except ImportError:
    np = None
try:
    import pandas as pd
except ImportError:
    pd = None
try:
    import tensorflow as tf
except ImportError:
    tf = None
try:
    import matplotlib.pyplot as plt
except ImportError:
    plt = None
try:
    import requests
except ImportError:
    requests = None
try:
    import flask
except ImportError:
    flask = None
from decimal import Decimal
from collections.abc import Mapping, Iterable
from enum import Enum
try:
    from scipy.sparse import csr_matrix
except ImportError:
    csr_matrix = None

""" test all the things! """


if flask:
    app = flask.Flask(__name__)

    @app.route('/test', methods=['GET', 'POST'])
    def test_route():
        return json.flat_dumps(flask.request)


class MyEnum(Enum):
    A = 1
    B = 2


class CustomObject:
    def _asdict(self):
        return {'giga_key': 'giga_value'}


class Mappy:
    def __init__(self): self.data = {'pie': 'thawn'}
    def __iter__(self): return iter(self.data)
    def __getitem__(self, key): return self.data[key]
    def items(self): return self.data.items()


class CustomJSONEncoderTestCase(unittest.TestCase):

    maxDiff = None

    @unittest.skipIf(flask is None, "flask is not installed.")
    def setUp(self):
        self.app = app.test_client()

    @unittest.skipIf(flask is None, "flask is not installed.")
    def test_flask_request_get(self):
        response = self.app.get('/test', headers={'User-Agent': 'UnitTest'}, query_string={'param': 'value'})


        print('\n\n\n\n\n\n\n')

        print(type(response).__name__)

        print('\n\n\n\n\n\n\n')


        actual = json.loads(response.data)
        expected = {'url': 'http://localhost/test?param=value', 'http_method': 'GET', 'headers': {'User-Agent': 'UnitTest', 'Host': 'localhost'}, 'user_agent': 'UnitTest', 'ip_address': '127.0.0.1', 'body': {'param': 'value'}}
        self.assertEqual(actual, expected)

    @unittest.skipIf(flask is None, "flask is not installed.")
    def test_flask_request_post(self):
        response = self.app.post('/test', headers={'User-Agent': 'UnitTest', 'Content-Type': 'application/json'}, json={'key': 'value'})
        actual = json.loads(response.data)
        expected = {'url': 'http://localhost/test', 'http_method': 'POST', 'headers': {'User-Agent': 'UnitTest', 'Host': 'localhost', 'Content-Type': 'application/json', 'Content-Length': '16'}, 'user_agent': 'UnitTest', 'ip_address': '127.0.0.1', 'body': {'key': 'value'}}
        self.assertEqual(actual, expected)

    def test_date(self):

        date = datetime.date(2023, 4, 1)
        self.assertEqual(json.dumps(date), '"2023-04-01"')

    def test_datetime(self):

        dt = datetime.datetime(2023, 4, 1, 12, 0)
        self.assertEqual(json.dumps(dt), '"2023-04-01T12:00:00"')

    def test_decimal(self):

        dec = Decimal('12.34')
        self.assertEqual(json.dumps(dec), '12.34')

    def test_uuid(self):

        u = uuid.uuid4()
        self.assertEqual(json.dumps(u), '"' + str(u) + '"')

    def test_mapping(self):

        m = {'a': 1, 'b': 2}
        self.assertEqual(json.dumps(m), '{\n    "a": 1,\n    "b": 2\n}')

    def test_iterable(self):

        l = [1, 2, 3]
        self.assertEqual(json.dumps(l), '[\n    1,\n    2,\n    3\n]')

    def test_bytes(self):

        b = b'hello'
        self.assertEqual(json.dumps(b), '"hello"')

    def test_set(self):

        s = {1, 2, 3}
        self.assertEqual(json.dumps(s), '[\n    1,\n    2,\n    3\n]')

    def test_complex(self):

        c = 1 + 2j
        self.assertEqual(json.dumps(c), '"1 + 2i"')

        c = 0 + 1j
        self.assertEqual(json.dumps(c), '"0 + 1i"')

        c = 4.1 - 0j
        self.assertEqual(json.dumps(c), '"4.1 + 0i"')

        c = 3.14 - 2.4j
        self.assertEqual(json.dumps(c), '"3.14 - 2.4i"')

        c = 0 + 0j
        self.assertEqual(json.dumps(c), '"0 + 0i"')

        c = 2j
        self.assertEqual(json.dumps(c), '"0 + 2i"')

    def test_asdict(self):

        obj = CustomObject()
        self.assertEqual(json.dumps(obj), '{\n    "giga_key": "giga_value"\n}')

    def test_custom(self):

        mappy = Mappy()
        self.assertEqual(json.dumps(mappy), '{\n    "pie": "thawn"\n}')

    @unittest.skipIf(np is None, "NumPy is not installed.")
    def test_numpy(self):

        numpy_array = np.array([1, 2, 3])
        self.assertEqual(json.flat_dumps(numpy_array), '[1, 2, 3]')

        numpy_int = np.int32(10)
        self.assertEqual(json.flat_dumps(numpy_int), '10')

        numpy_dtype = numpy_array.dtype
        self.assertEqual(json.flat_dumps(numpy_dtype), '"int64"')

        # deprecated; use ndarray
        # numpy_matrix = np.matrix([[1, 2], [3, 4]])
        # self.assertEqual(json.flat_dumps(numpy_matrix), '[[1, 2], [3, 4]]')

        numpy_masked_array = np.ma.masked_array([1, 2], mask=[False, True])
        self.assertEqual(json.flat_dumps(numpy_masked_array), '{"data": [1, 2], "mask": [false, true]}')

        numpy_recarray = np.recarray((2,), dtype=[('x', int), ('y', float)])
        numpy_recarray[:] = [(1, 1.0), (2, 2.0)]
        self.assertEqual(json.flat_dumps(numpy_recarray), '[{"x": 1, "y": 1.0}, {"x": 2, "y": 2.0}]')

    @unittest.skipIf(pd is None, "Pandas is not installed.")
    def test_pandas(self):

        df = pd.DataFrame({'a': [1, 2], 'b': [3, 4]})
        self.assertEqual(json.flat_dumps(df), '[{"a": 1, "b": 3}, {"a": 2, "b": 4}]')

        series = pd.Series([1, 2, 3], index=['a', 'b', 'c'])
        self.assertEqual(json.flat_dumps(series), '{"a": 1, "b": 2, "c": 3}')

        index = pd.Index([1, 2, 3])
        self.assertEqual(json.flat_dumps(index), '[1, 2, 3]')

    @unittest.skipIf(torch is None, "PyTorch is not installed.")
    def test_pytorch(self):
        tensor = torch.tensor([[1, 2], [3, 4]])
        self.assertEqual(json.flat_dumps(tensor), '[[1, 2], [3, 4]]')

    @unittest.skipIf(tf is None, "TensorFlow is not installed.")
    def test_tensorflow(self):
        tensor = tf.constant([[1, 2], [3, 4]])
        self.assertEqual(json.flat_dumps(tensor), '[[1, 2], [3, 4]]')

    @unittest.skipIf(plt is None, "MatPlotLib is not installed.")
    def test_matplotlib(self):
        fig, ax = plt.subplots()
        ax.plot([1, 2, 3], [4, 5, 6])
        self.assertEqual(json.flat_dumps(ax), '[{"x": [1, 2, 3], "y": [4, 5, 6]}]')

    @unittest.skipIf(csr_matrix is None, "SciPy is not installed.")
    def test_scipy(self):
        sparse_matrix = csr_matrix([[1, 0, 0], [0, 2, 0], [0, 0, 3]])
        self.assertEqual(json.flat_dumps(sparse_matrix), '[[1, 0, 0], [0, 2, 0], [0, 0, 3]]')

    @unittest.skipIf(requests is None, "requests is not installed.")
    def test_requests(self):
        response = requests.models.Response()
        response._content = b'{"key": "value"}'
        response.status_code = 200
        response.reason = 'OK'
        response.headers = requests.structures.CaseInsensitiveDict({'Content-Type': 'application/json'})
        self.assertEqual(json.flat_dumps(response), '{"key": "value", "status_code": 200, "reason": "OK", "headers": {"Content-Type": "application/json"}}')

    def test_enum(self):
        self.assertEqual(json.dumps(MyEnum.A), '1')

    def test_memory_view_serialization(self):
        data = bytearray(b'hello world')
        mem_view = memoryview(data)
        serialized_data = json.dumps(mem_view)
        self.assertEqual(serialized_data, '"hello world"')

    def test_fallback_to_str(self):
        obj = object()
        self.assertTrue(isinstance(json.dumps(obj), str))


if __name__ == '__main__':
    unittest.main(verbosity=2)
